#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.printChanges = void 0;
const chalk = require("chalk");
const tree_1 = require("nx/src/generators/tree");
const nx_plugin_1 = require("nx/src/utils/nx-plugin");
const node_path_1 = require("node:path");
const node_process_1 = require("node:process");
const commander_1 = require("commander");
const OpenAPIParser = require("@readme/openapi-parser");
async function main({ argv }) {
    var _a;
    console.info(`Executing cli...`);
    const { options } = createProgram(argv);
    const specs = (_a = options.specs) !== null && _a !== void 0 ? _a : (await resolveEPCCOpenApiSpecs());
    console.log('program options: ', options);
    console.log(`${chalk.green('Started generating clients...')}`);
    for (const spec of specs) {
        console.log(`${chalk.green('Using spec: ')} ${typeof spec === 'string' ? spec : spec.info.title}`);
    }
    const { workspace, dryRun } = options;
    for (const spec of specs) {
        console.log(`${chalk.green(`Starting generating client based on spec`)} ${typeof spec === 'string' ? spec : spec.info.title}`);
        await generateClient(spec, workspace, dryRun);
    }
    console.log(`${chalk.green('Completed generating clients!')}`);
}
async function resolveEPCCOpenApiSpecs() {
    const module = await Promise.resolve().then(() => require('@field123/epcc-openapi'));
    return Promise.all(Object.keys(module).map(key => {
        const spec = module[key].default;
        console.log(key, spec);
        return OpenAPIParser.dereference(spec);
    }));
}
function printChanges(fileChanges) {
    fileChanges.forEach(f => {
        if (f.type === 'CREATE') {
            console.log(`${chalk.green('CREATE')} ${f.path}`);
        }
        else if (f.type === 'UPDATE') {
            console.log(`${chalk.white('UPDATE')} ${f.path}`);
        }
        else if (f.type === 'DELETE') {
            console.log(`${chalk.yellow('DELETE')} ${f.path}`);
        }
    });
}
exports.printChanges = printChanges;
function createProgram(argv) {
    const program = new commander_1.Command();
    const cli = program
        .description('EPCC Typescript client generating cli')
        .option('-w, --workspace [value]', 'Workspace to generate clients in.', 'clients')
        .option('-n, --dry-run', 'Dry run will prevent outputting of any files.')
        .option('-s, --specs [specs...]', 'Optional paths to open api v3 specifications')
        .parse(argv);
    const options = program.opts();
    return { cli, options };
}
async function generateClient(spec, workspace, dryRun) {
    const host = new tree_1.FsTree(workspace, true);
    (0, nx_plugin_1.registerPluginTSTranspiler)();
    const module = await Promise.resolve().then(() => require('./generators/service-client-generator/generator'));
    const implementation = module.default;
    const task = await implementation(host, {
        spec: typeof spec === 'string' ? (0, node_path_1.resolve)((0, node_process_1.cwd)(), spec) : spec
    });
    host.lock();
    const changes = host.listChanges();
    printChanges(changes);
    if (!dryRun) {
        (0, tree_1.flushChanges)(workspace, changes);
        if (task) {
            await task();
        }
    }
    else {
        console.warn(`\nNOTE: The "dryRun" flag means no changes were made.`);
    }
}
console.log(process.argv);
main({ argv: process.argv });
